﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Benchmark.Problem
{
    [TestClass]
    public class CountOdds : Problem<long[], long>
    {
        public const string ProblemFileName = "examples-count-odds.csv";

        public CountOdds()
        {
            this.Examples = this.GetExamples(ProblemFileName, ExampleConverter.Integers, ExampleConverter.Integer);
        }

        [TestMethod]
        public void RandomWalk()
        {
            var interpreter = new PushGPInterpreter(new Configuration
            {
                EvalPushLimit = 10000
            });

            var limit = 1000;
            var iterations = 3000;
            var best = double.MaxValue;

            for (var i = 0; i < iterations; i++)
            {
                var program = interpreter.CodeGenerator.RandomProgram(limit);

                var results = new List<long>();

                foreach (var example in this.Examples)
                {
                    if (example.Input.Length > 0)
                    {
                        interpreter.IntegerStack.Push(example.Input);
                    }

                    interpreter.Interpret(program);

                    if (!interpreter.IntegerStack.IsEmpty)
                    {
                        var value = Math.Abs(example.Output - interpreter.IntegerStack.Top);

                        results.Add(Math.Min(value, 1000));
                    }
                }


                if (!results.Any())
                    continue;

                var avg = results.Average();

                if (avg < best)
                {
                    best = avg;
                }

                interpreter.Clear();
            }

            Debug.WriteLine("Best: {0}", best);
        }
    }
}
